unit GameOfLife;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type
    {GameOfLife}
    multiArray = Array of Array of Boolean;
    TGameOfLife = class
	private
        mArray: Array of Array of Boolean;
        mGeneration: Integer;
        mChanged: Boolean;
        mRunning: Boolean;

        procedure init();
    public
        procedure reset(y, x: Integer);
        procedure nextRound();
        function cntNeighbour(posY, posX: Integer): Integer;
        function isRunning(): Boolean;
        function hasChanged(): Boolean;
        function getGeneration(): Integer;
        function getArray(): multiArray;
        function getX(): Integer;
        function getY(): Integer;
    end;

implementation

procedure TGameOfLife.init();
var
    x, y: Integer;
begin
    for y:= 0 to High(self.mArray) do
    begin
        for x:= 0 to High(self.mArray[y]) do
        begin
            randomize;
            self.mArray[y][x]:= random(10) > 5;
        end;
    end;
end;

procedure TGameOfLife.reset(y, x: Integer);
var
    i: Integer;
begin
    SetLength(self.mArray,y);

    for i:= 0 to y-1 do
    begin
        SetLength(self.mArray[i],x);
    end;

    self.mGeneration:= 0;
    self.mRunning:= False;
    self.mChanged:= True;
end;

function TGameOfLife.cntNeighbour(posY, posX: Integer): Integer;
var
    res, y, x, relY, relX: Integer;
begin
    res:= 0;

    for y:= -1 to 1 do
    begin
        relY:= (posY + y) mod (High(self.mArray) + 1);

        if relY < 0 then
            relY:= relY + (High(self.mArray) + 1);

        for x:= -1 to 1 do
        begin
            if (x <> 0) or (y <> 0) then
            begin
                relX:= (posX + x) mod (High(self.mArray[y]) + 1);

                if relX < 0 then
                    relX:= relX + (High(self.mArray[y]) + 1);

                if self.mArray[relY][relX] then
                   res:= res + 1;
            end;
        end;
    end;

    cntNeighbour:= res;
end;

procedure TGameOfLife.nextRound();
var
    tmp: Array of Array of Boolean;
    y, x, res: Integer;
begin
    SetLength(tmp,High(self.mArray) + 1);

    for y:= 0 to High(self.mArray) do
    begin
        SetLength(tmp[y],High(self.mArray[y]) + 1);

        for x:= 0 to High(self.mArray[y]) do
        begin
            res:= self.cntNeighbour(y,x);

            if self.mArray[y][x] then
            begin
                if (res < 2) or (res > 3) then
                    tmp[y][x]:= False
                else
                    tmp[y][x]:= True;
            end
            else
            begin
                if res = 3 then
                    tmp[y][x]:= True
                else
                	tmp[y][x]:= False;
            end;
        end;
    end;

    self.mArray:= tmp;
    self.mGeneration:= self.mGeneration + 1;
    self.mChanged:= True;
end;

function TGameOfLife.getGeneration(): Integer;
begin
    getGeneration:= self.mGeneration;
end;

function TGameOfLife.isRunning(): Boolean;
begin
    isRunning:= self.mRunning;
end;

function TGameOfLife.hasChanged(): Boolean;
begin
    if self.mRunning then
        hasChanged:= True
    else
        if self.mChanged then
            hasChanged:= True
        else
            hasChanged:= False;

    self.mChanged:= False;
end;

function TGameOfLife.getArray(): multiArray;
begin
    getArray:= self.mArray;
end;

function TGameOfLife.getX(): Integer;
begin
    getX:= High(self.mArray[0]) + 1;
end;

function TGameOfLife.getY(): Integer;
begin
    getY:= High(self.mArray) + 1;
end;

end.

